#include <sys/os16.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <libgen.h>
//----------------------------------------------------------------------
static void usage (void);
//----------------------------------------------------------------------
int
main (int argc, char *argv[], char *envp[])
{
    char       *source;
    char       *destination;
    char       *new_destination;
    struct stat file_status;
    int         dest_is_a_dir = 0;
    int         a;                      // Argument index.
    char        path[PATH_MAX];
    //
    // There must be at least two arguments, plus the program name.
    //
    if (argc < 3)
      {
        usage ();
        return (1);
      }
    //
    // Select the last argument as the destination.
    //
    destination = argv[argc-1];
    //
    // Check if it is a directory and save it in a flag.
    //
    if (stat (destination, &file_status) == 0)
      {
        if (S_ISDIR (file_status.st_mode))
          {
            dest_is_a_dir = 1;
          }
      }
    //
    // If there are more than two arguments, verify that the last
    // one is a directory.
    //
    if (argc > 3)
      {
        if (!dest_is_a_dir)
          {
            usage ();
            fprintf (stderr, "The destination \"%s\" ",
                     destination);
            fprintf (stderr, "is not a directory!\n");
            return (1);
          }
      }
    //
    // Scan the arguments, excluded the last, that is the destination.
    //
    for (a = 1; a < (argc - 1); a++)
      {
        //
        // Source.
        //
        source = argv[a];
        //
        // Verify access permissions.
        //
        if (access (source, R_OK) < 0)
          {
            perror (source);
            continue;
          }
        //
        // Destination.
        //
        // If it is a directory, the destination path
        // must be corrected.
        //
        if (dest_is_a_dir)
          {
            path[0] = 0;
            strcat (path, destination);
            strcat (path, "/");
            strcat (path, basename (source));
            //
            // Update the destination path.
            //
            new_destination = path;
          }
        else
          {
            new_destination = destination;
          }
        //
        // Check if destination file exists.
        //
        if (stat (new_destination, &file_status) == 0)
          {
            fprintf (stderr, "The destination file, \"%s\", ",
                     new_destination);
            fprintf (stderr, "already exists!\n");
            continue;
          }
        //
        // Everything is ready for the link.
        //
        if (link (source, new_destination) < 0)
          {
            perror (new_destination);
            continue;
          }
      }
    //
    // All done.
    //
    return (0);
}
//----------------------------------------------------------------------
static void
usage (void)
{
    fprintf (stderr, "Usage: ln OLD_NAME NEW_NAME\n");
    fprintf (stderr, "       ln FILE... DIRECTORY\n");
}
