#include <sys/os16.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <libgen.h>
//----------------------------------------------------------------------
static int  mkdir_parents (const char *path, mode_t mode);
static void usage         (void);
//----------------------------------------------------------------------
int
main (int argc, char *argv[], char *envp[])
{
    sysmsg_uarea_t msg;
    int            status;
    mode_t         mode     = 0;
    int            m;           // Index inside mode argument.
    int            digit;
    char         **dir;
    int            d;           // Directory index.
    int            option_p = 0;
    int            option_m = 0;
    int            opt;
    extern char   *optarg;
    extern int     optind;
    extern int     optopt;
    //
    // There must be at least an argument, plus the program name.
    //
    if (argc < 2)
      {
        usage ();
        return (1);
      }
    //
    // Check for options, starting from `p'. The `dir' pointer is used
    // to calculate the argument pointer to the first directory [1].
    // The macro-instruction `max()' is declared inside <sys/os16.h>
    // and does the expected thing.
    //
    while ((opt = getopt (argc, argv, ":pm:")) != -1)
      {
        switch (opt)
          {
            case 'm':
                option_m = 1;
                for (m = 0; m < strlen (optarg); m++)
                  {
                    digit = (optarg[m] - '0');
                    if (digit < 0 || digit > 7)
                      {
                        usage ();
                        return (2);
                      }
                    mode = mode * 8 + digit;
                  }
                break;
            case 'p':
                option_p = 1;
                break;
            case '?':
                printf ("Unknown option -%c.\n", optopt);
                usage ();
                return (1);
                break;
            case ':':
                printf ("Missing argument for option -%c\n", optopt);
                usage ();
                return (2);
                break;
            default:
                printf ("Getopt problem: unknown option %c\n", opt);
                return (3);
          }
      }
    //
    dir = argv + optind;
    //
    // Check if the mode is to be set to a default value.
    //
    if (!option_m)
      {
        //
        // Default mode.
        //
        sys (SYS_UAREA, &msg, (sizeof msg));
        mode = 0777 & ~msg.umask;
      }
    //
    // Directory creation.
    //
    for (d = 0; dir[d] != NULL; d++)
      {
        if (option_p)
          {
            status = mkdir_parents (dir[d], mode);
            if (status != 0)
             {
                perror (dir[d]);
                return (3);
              }
          }
        else
          {
            status = mkdir (dir[d], mode);
            if (status != 0)
              {
                perror (dir[d]);
                return (4);
              }
          }
      }
    //
    // All done.
    //
    return (0);
}
//----------------------------------------------------------------------
static int
mkdir_parents (const char *path, mode_t mode)
{
    char        path_copy[PATH_MAX];
    char       *path_parent;
    struct stat fst;
    int         status;
    //
    // Check if the path is empty.
    //
    if (path == NULL || strlen (path) == 0)
      {
        //
        // Recursion ends here.
        //
        return (0);
      }
    //
    // Check if it does already exists.
    //
    status = stat (path, &fst);
    if (status == 0 && fst.st_mode & S_IFDIR)
      {
        //
        // The path exists and is a directory.
        //
        return (0);
      }
    else if (status == 0 && !(fst.st_mode & S_IFDIR))
      {
        //
        // The path exists but is not a directory.
        //
        errno = ENOTDIR;        // Not a directory.
        return (-1);
      }
    //
    // Get the directory path.
    //
    strncpy (path_copy, path, PATH_MAX);
    path_parent = dirname (path_copy);
    //
    // If it is `.', or `/', the recursion is terminated.
    //
    if (strncmp (path_parent, ".", PATH_MAX) == 0  ||
        strncmp (path_parent, "/", PATH_MAX) == 0)
      {
        return (0);
      }
    //
    // Otherwise, continue the recursion.
    //
    status = mkdir_parents (path_parent, mode);
    if (status != 0)
      {
        return (-1);
      }
    //
    // Previous directories are there: create the current one.
    //
    status = mkdir (path, mode);
    if (status)
      {
        perror (path);
        return (-1);
      }

    return (0);
}
//----------------------------------------------------------------------
static void
usage (void)
{
    fprintf (stderr, "Usage: mkdir [-p] [-m OCTAL_MODE] DIR...\n");
}
