#ifndef _KERNEL_IBM_I86_H
#define _KERNEL_IBM_I86_H 1

#include <stdint.h>
#include <size_t.h>
#include <kernel/memory.h>
#include <sys/types.h>
//----------------------------------------------------------------------
#define IBM_I86_VIDEO_MODE   0x02
#define IBM_I86_VIDEO_PAGES  4

#define IBM_I86_VIDEO_COLUMNS	80
#define IBM_I86_VIDEO_ROWS	25
#define IBM_I86_VIDEO_ADDRESS   0xB8000L, 0xB9000L, 0xBA000L, 0xBB000L
//----------------------------------------------------------------------
void     _int10_00 (uint16_t video_mode);
void     _int10_02 (uint16_t page, uint16_t position);
void     _int10_05 (uint16_t page);
uint16_t _int12    (void);
uint16_t _int13_00 (uint16_t drive);
uint16_t _int13_02 (uint16_t drive, uint16_t sectors,
                    uint16_t cylinder, uint16_t head,
                    uint16_t sector, void *buffer);
uint16_t _int13_03 (uint16_t drive, uint16_t sectors,
                    uint16_t cylinder, uint16_t head,
                    uint16_t sector, void *buffer);
uint16_t _int16_00 (void);
uint16_t _int16_01 (void);
uint16_t _int16_02 (void);

#define int10_00(video_mode)     (_int10_00 ((uint16_t) video_mode))
#define int10_02(page, position) (_int10_02 ((uint16_t) page, \
                                             (uint16_t) position))
#define int10_05(page)           (_int10_05 ((uint16_t) page))
#define int12()                  ((unsigned int) _int12 ())

#define int13_00(drive)          ((unsigned int) \
                                  _int13_00 ((uint16_t) drive))
#define int13_02(drive, sectors, cylinder, head, sector, buffer) \
                                 ((unsigned int) \
                                  _int13_02 ((uint16_t) drive, \
                                             (uint16_t) sectors, \
                                             (uint16_t) cylinder, \
                                             (uint16_t) head, \
                                             (uint16_t) sector, \
                                             buffer))
#define int13_03(drive, sectors, cylinder, head, sector, buffer) \
                                 ((unsigned int) \
                                  _int13_03 ((uint16_t) drive, \
                                             (uint16_t) sectors, \
                                             (uint16_t) cylinder, \
                                             (uint16_t) head, \
                                             (uint16_t) sector, \
                                             buffer))
#define int16_00()               ((unsigned int) _int16_00 ())
#define int16_01()               ((unsigned int) _int16_01 ())
#define int16_02()               ((unsigned int) _int16_02 ())
//----------------------------------------------------------------------
uint16_t _in_8   (uint16_t port);
uint16_t _in_16  (uint16_t port);
void     _out_8  (uint16_t port, uint16_t value);
void     _out_16 (uint16_t port, uint16_t value);

#define in_8(port)           ((unsigned int) _in_8 ((uint16_t) port))
#define in_16(port)          ((unsigned int) _in_16 ((uint16_t) port))
#define out_8(port, value)   (_out_8 ((uint16_t) port, \
                                      (uint16_t) value))
#define out_16(port, value)  (_out_16 ((uint16_t) port, \
                                       (uint16_t) value))
//----------------------------------------------------------------------
void _cli     (void);
void _sti     (void);

#define cli() (_cli ())
#define sti() (_sti ())
//----------------------------------------------------------------------
void irq_on  (unsigned int irq);
void irq_off (unsigned int irq);
//----------------------------------------------------------------------
void     _ram_copy      (segment_t org_seg, offset_t org_off,
                         segment_t dst_seg, offset_t dst_off,
                         uint16_t size);

#define ram_copy(org_seg, org_off, dst_seg, dst_off, size) \
                        (_ram_copy ((uint16_t) org_seg, \
                                    (uint16_t) org_off, \
                                    (uint16_t) dst_seg, \
                                    (uint16_t) dst_off, \
                                    (uint16_t) size))
//----------------------------------------------------------------------
void con_select     (int console);
void con_putc       (int console, int c);
void con_scroll     (int console);
int  con_char_wait  (void);
int  con_char_read  (void);
int  con_char_ready (void);
void con_init       (void);
//----------------------------------------------------------------------
#define DSK_MAX                   4
#define DSK_SECTOR_SIZE         512     // Fixed!

typedef struct {
    unsigned int  bios_drive;
    unsigned int  cylinders;
    unsigned int  heads;
    unsigned int  sectors;
    unsigned int  retry;
} dsk_t;

typedef struct {
    unsigned int  cylinder;
    unsigned int  head;
    unsigned int  sector;
} dsk_chs_t;
//----------------------------------------------------------------------
extern dsk_t dsk_table[DSK_MAX];
//----------------------------------------------------------------------
void   dsk_setup         (void);
int    dsk_reset         (int drive);
void   dsk_sector_to_chs (int drive, unsigned int sector,
                          dsk_chs_t *chs);
int    dsk_read_sectors  (int drive, unsigned int start_sector,
                          void *buffer, unsigned int n_sectors);
int    dsk_write_sectors (int drive, unsigned int start_sector,
                          void *buffer, unsigned int n_sectors);
size_t dsk_read_bytes    (int drive, off_t offset,
                          void *buffer, size_t count);
size_t dsk_write_bytes   (int drive, off_t offset,
                          void *buffer, size_t count);
//----------------------------------------------------------------------

#endif
