#include <kernel/ibm_i86.h>
#include <kernel/k_libc.h>
#include <sys/os16.h>
#include <sys/types.h>
#include <stdint.h>
//----------------------------------------------------------------------
void
con_putc (int console, int c)
{
    static int      cursor[IBM_I86_VIDEO_PAGES];
    static addr_t   address[] = {IBM_I86_VIDEO_ADDRESS};
           addr_t   address_destination;
           size_t   size_screen;
           size_t   size_row;
           uint16_t cell;
           uint16_t attribute = 0x0700;
           int      cursor_row;
           int      cursor_column;
           int      cursor_combined;

    if (console < 0 || console >= IBM_I86_VIDEO_PAGES)
      {
        //
        // No such console.
        //
        return;
      }
    //
    // Calculate sizes.
    //
    size_row    = IBM_I86_VIDEO_COLUMNS;
    size_screen = size_row * IBM_I86_VIDEO_ROWS;
    //
    // See if it is a special character, or if the cursor position
    // requires a scroll up.
    //
    if (c == '\n')
      {
        con_scroll (console);
        cursor[console] = (size_screen - size_row);
      }
    else if (c == '\b')
      {
        cursor[console]--;
        if (cursor[console] < 0)
          {
            cursor[console] = 0;
          }
      }
    else if (cursor[console] == (size_screen -1))
      {
        //
        // Scroll up.
        //
        con_scroll (console);
        //
        cursor[console] -= size_row;
      }
    //
    // If it is not a control character, print it.
    //
    if (c != '\n' && c != '\b')
      {
        //
        // Write the character.
        //
        address_destination  = address[console];
        address_destination += (cursor[console] * 2);
        cell = (attribute | (c & 0x00FF));
        //
        mem_write (address_destination, &cell, sizeof (uint16_t));
        //
        // and an extra space after it (to be able to show the cursor).
        //
        cell = (attribute | ' ');
        address_destination  += 2;
        mem_write (address_destination, &cell, sizeof (uint16_t));
        //
        //
        //
        cursor[console]++;
      }
    //
    // Update the cursor position on screen.
    //
    cursor_row      = cursor[console] / size_row;
    cursor_column   = cursor[console] % size_row;
    cursor_combined = (cursor_row << 8) | cursor_column;
    //
    // Set cursor position.
    //
    int10_02 (console, cursor_combined);
}
