#include <kernel/memory.h>
#include <kernel/ibm_i86.h>
#include <sys/os16.h>
#include <errno.h>
//----------------------------------------------------------------------
static int mb_block_status (int block);
//----------------------------------------------------------------------
int
mb_alloc_size (size_t size, memory_t *allocated)
{
    unsigned int bsize;
    unsigned int i;
    unsigned int j;
    unsigned int found = 0;
    addr_t       alloc_addr;
    ssize_t      alloc_size;

    if (size == 0)
      {
        //
        // Zero means the maximum size.
        //
        bsize = 0x10000L / MEM_BLOCK_SIZE;
      }
    else if (size % MEM_BLOCK_SIZE)
      {
        bsize = size / MEM_BLOCK_SIZE + 1;
      }
    else
      {
        bsize = size / MEM_BLOCK_SIZE;
      }

    for (i = 0; i < (MEM_MAX_BLOCKS - bsize) && !found; i++)
      {
        for (j = 0; j < bsize; j++)
          {
            found = !mb_block_status (i+j);
            if (!found)
              {
                i += j;
                break;
              }
          }
      }

    if (found && (j == bsize))
      {
        alloc_addr  = i - 1;
        alloc_addr *= MEM_BLOCK_SIZE;
        alloc_size  = bsize * MEM_BLOCK_SIZE;
        alloc_size  = mb_alloc (alloc_addr, (size_t) alloc_size);
        if (alloc_size <= 0)
          {
            errset (ENOMEM);
            return (-1);
          }
        else if (alloc_size < size)
          {
            mb_free (alloc_addr, (size_t) alloc_size);
            errset (ENOMEM);
            return (-1);
          }
        else
          {
            allocated->address = alloc_addr;
            allocated->segment = alloc_addr / 16;
            allocated->size    = (size_t) alloc_size;
          }
        return (0);
      }
    else
     {
        errset (ENOMEM);
        return (-1);
     }
}
//----------------------------------------------------------------------
static int
mb_block_status (int block)
{
    int i         = block / 16;
    int j         = block % 16;
    uint16_t mask = 0x8000 >> j;
    return ((int) (mb_table[i] & mask));
}
