#include <kernel/memory.h>
#include <kernel/ibm_i86.h>
#include <sys/os16.h>
#include <kernel/k_libc.h>
//----------------------------------------------------------------------
static int mb_block_set0   (int block);
//----------------------------------------------------------------------
void
mb_free (addr_t address, size_t size)
{
    unsigned int bstart;
    unsigned int bsize;
    unsigned int bend;
    unsigned int i;
    addr_t       block_address;
    if (size == 0)
      {
        //
        // Zero means the maximum size.
        //
        bsize  = 0x10000L / MEM_BLOCK_SIZE;
      }
    else
      {
        bsize  = size / MEM_BLOCK_SIZE;
      }

    bstart = address / MEM_BLOCK_SIZE;

    if (size % MEM_BLOCK_SIZE)
      {
        bend = bstart + bsize;
      }
    else
      {
        bend = bstart + bsize - 1;
      }

    for (i = bstart; i <= bend; i++)
      {
        if (mb_block_set0 (i))
          {
            ;
          }
        else
          {
            block_address = i;
            block_address *= MEM_BLOCK_SIZE;
            k_printf ("Kernel alert: mem block %04x, at address ", i);
            k_printf ("%05lx, already released!\n", block_address);
          }
      }
}
//----------------------------------------------------------------------
static int
mb_block_set0 (int block)
{
    int i         = block / 16;
    int j         = block % 16;
    uint16_t mask = 0x8000 >> j;
    if (mb_table[i] & mask)
      {
        mb_table[i] = mb_table[i] & ~mask;
        return (1);
      }
    else
      {
        return (0);     // The block is already set to 0 inside the map!
      }
}
