.extern _proc_scheduler
.extern _sysroutine
.global __ksp
.global __clock_ticks
.global __clock_seconds
.global isr_1C
.global isr_80
;-----------------------------------------------------------------------
; The kernel code segment starts at 0x10500 (segment 0x1050).
; The kernel data segments start at 0x00500 (segment 0x0050).
; To switch to the kernel data segments, DS, ES and SS are set to
; 0x0050. To identify the kernel context, the DS register is checked:
; if it is equal to 0x0050, it is the kernel.
;-----------------------------------------------------------------------
.data
;-----------------------------------------------------------------------
.align 2
proc_ss_0:       .word 0x0000
proc_sp_0:       .word 0x0000
proc_ss_1:       .word 0x0000
proc_sp_1:       .word 0x0000
proc_syscallnr:  .word 0x0000
proc_msg_offset: .word 0x0000
proc_msg_size:   .word 0x0000
__ksp:           .word 0x0000
__clock_ticks:
ticks_lo:        .word 0x0000
ticks_hi:        .word 0x0000
__clock_seconds:
seconds_lo:      .word 0x0000
seconds_hi:      .word 0x0000
;-----------------------------------------------------------------------
.text
;-----------------------------------------------------------------------
; IRQ 0: "timer".
; IRQ 0 is associated to INT 8, and after the BIOS work is done,
; INT 1C is called. Standard INT 1C has nothing to do, but is
; useful to call extra work for the timer. As the original BIOS
; interrupts are used, the INT 1C is reprogrammed, keeping intact
; the Standard INT 8.
;-----------------------------------------------------------------------
.align 2
isr_1C:
    ;-------------------------------------------------------------------
    ; Inside the process stack, the CPU already put:
    ;
    ;       [omissis]
    ;       push flags
    ;       push cs
    ;       push ip
    ;-------------------------------------------------------------------
    ;
    ; Save into process stack:
    ;
    push   es   ; extra segment
    push   ds   ; data segment
    push   di   ; destination index
    push   si   ; source index
    push   bp   ; base pointer
    push   bx   ; BX
    push   dx   ; DX
    push   cx   ; CX
    push   ax   ; AX
    ;
    ; Set the data segments to the kernel data area,
    ; so that the following variables can be accessed.
    ;
    mov ax, #0x0050  ; DS and ES.
    mov ds, ax       ;
    mov es, ax       ;
    ;
    ; Increment time counters, to keep time.
    ;
    add ticks_lo, #1    ; Clock ticks counter.
    adc ticks_hi, #0    ;
    ;
    mov dx, ticks_hi    ;
    mov ax, ticks_lo    ; DX := ticks % 18
    mov cx, #18         ;
    div cx              ;
    mov ax, #0          ; If the ticks value can be divided by 18,
    cmp ax, dx          ; the seconds is incremented by 1.
    jnz L1              ;
    add seconds_lo, #1  ;
    adc seconds_hi, #0  ; 
    ;
L1: ; Save process stack registers into kernel data segment.
    ;
    mov proc_ss_0, ss   ; Save process stack segment.
    mov proc_sp_0, sp   ; Save process stack pointer.
    ;
    ; Check if it is already in kernel mode.
    ;
    mov dx, proc_ss_0
    mov ax, #0x0050     ; Kernel data area.
    cmp dx, ax
    je L2
    ;
    ; If we are here, a user process was interrupted.
    ; Switch to the kernel stack.
    ;
    mov ax, #0x0050     ; Kernel data area.
    mov ss, ax
    mov sp, __ksp
    ;
    ; Call the scheduler.
    ;
    push #proc_ss_0     ; &proc_ss_0
    push #proc_sp_0     ; &proc_sp_0
    call _proc_scheduler
    add  sp, #2
    add  sp, #2
    ;
    ; Restore process stack registers from kernel data segment.
    ;
    mov ss, proc_ss_0   ; Restore process stack segment.
    mov sp, proc_sp_0   ; Restore process stack pointer.
    ;
L2: ; Restore from process stack:
    ;
    pop    ax
    pop    cx
    pop    dx
    pop    bx
    pop    bp
    pop    si
    pop    di
    pop    ds
    pop    es
    ;
    ; Return from interrupt: will restore CS:IP and FLAGS
    ; from process stack.
    ;
    iret
;-----------------------------------------------------------------------
; Syscall.
;-----------------------------------------------------------------------
.align 2
isr_80:
    ;-------------------------------------------------------------------
    ; Inside the process stack, we already have:
    ;       push #message_size
    ;       push &message_structure     ; the relative address of it
    ;       push #syscall_number
    ;       push #back_address  ; made by a call to _sys function
    ;       push flags          ; made by int #0x80
    ;       push cs             ; made by int #0x80
    ;       push ip             ; made by int #0x80
    ;-------------------------------------------------------------------
    ;
    ; Save into process stack:
    ;
    push   es   ; extra segment
    push   ds   ; data segment
    push   di   ; destination index
    push   si   ; source index
    push   bp   ; base pointer
    push   bx   ; BX
    push   dx   ; DX
    push   cx   ; CX
    push   ax   ; AX
    ;
    ; Set the data segments to the kernel data area,
    ; so that the following variables can be accessed.
    ;
    mov ax, #0x0050     ; DS and ES.
    mov ds, ax          ;
    mov es, ax          ;
    ;
    ; Save process stack registers into kernel data segment.
    ;
    mov proc_ss_1, ss   ; Save process stack segment.
    mov proc_sp_1, sp   ; Save process stack pointer.
    ;
    ; Save some more data, from the system call.
    ;    
    mov bp, sp
    mov ax, +26[bp]
    mov proc_syscallnr,  ax
    mov ax, +28[bp]
    mov proc_msg_offset, ax
    mov ax, +30[bp]
    mov proc_msg_size,   ax
    ;
    ; Check if it is already the kernel stack.
    ;
    mov dx, ss
    mov ax, #0x0050     ; Kernel data area.
    cmp dx, ax
    jne L3
    ;
    ; It is already the kernel stack, so, the variable "_ksp" is
    ; aligned to current stack pointer. This way, the first syscall
    ; can work without having to set the "_ksp" variable to some
    ; reasonable value.
    ;
    mov __ksp, sp
    ;    
L3: ; Switch to the kernel stack.
    ;
    mov ax, #0x0050     ; Kernel data area.
    mov ss, ax
    mov sp, __ksp
    ;
    ; Call the external hardware interrupt handler.
    ;
    push proc_msg_size
    push proc_msg_offset
    push proc_syscallnr
    push #proc_ss_1     ; &proc_ss_1
    push #proc_sp_1     ; &proc_sp_1
    call _sysroutine
    add  sp, #2
    add  sp, #2
    add  sp, #2
    add  sp, #2
    add  sp, #2
    ;
    ; Restore process stack registers from kernel data segment.
    ;
    mov ss, proc_ss_1   ; Restore process stack segment.
    mov sp, proc_sp_1   ; Restore process stack pointer.
    ;
    ; Restore from process stack:
    ;
    pop    ax
    pop    cx
    pop    dx
    pop    bx
    pop    bp
    pop    si
    pop    di
    pop    ds
    pop    es
    ;
    ; Return from interrupt: will restore CS:IP and FLAGS
    ; from process stack.
    ;
    iret
