#include <kernel/proc.h>
#include <fcntl.h>
//----------------------------------------------------------------------
void
proc_dump_memory (pid_t pid, addr_t address, size_t size, char *name)
{
    int     fdn;
    char    buffer[SB_BLOCK_SIZE];
    ssize_t size_written;
    ssize_t size_written_total;
    ssize_t size_read;
    ssize_t size_read_total;
    ssize_t size_total;
    //
    // Dump the code segment to disk.
    //
    fdn = fd_open (pid, name, (O_WRONLY|O_CREAT|O_TRUNC),
                   (mode_t) (S_IFREG|00644));
    if (fdn < 0)
      {
        //
        // There is a problem: just let it go.
        //
        return;
      }
    //
    // Fix size: (size_t) 0 is equivalent to (ssize_t) 0x10000.
    //
    size_total = size;
    if (size_total == 0)
      {
        size_total = 0x10000;
      }
    //
    // Read the memory and write it to disk.
    //
    for (size_read = 0, size_read_total = 0;
         size_read_total < size_total;
         size_read_total += size_read, address += size_read)
      {
        size_read = mem_read (address, buffer, SB_BLOCK_SIZE);
        //
        for (size_written = 0, size_written_total = 0;
             size_written_total < size_read;
             size_written_total += size_written)
          {
            size_written = fd_write (pid, fdn,
                            &buffer[size_written_total],
                            (size_t) (size_read - size_written_total));
            //
            if (size_written < 0)
              {
                fd_close (pid, fdn);
                return;
              }
          }
      }
    fd_close (pid, fdn);
}
