#include <kernel/proc.h>
//----------------------------------------------------------------------
void
proc_sig_chld (pid_t parent, int sig)
{
    pid_t child;
    //
    // Please note that `sig' should be SIGCHLD and nothing else.
    // So, the following test, means to verify if the parent process
    // has received a SIGCHLD already.
    //
    if (proc_sig_status (parent, sig))
      {
        if (   (!proc_sig_ignore (parent, sig))
            && (proc_table[parent].status == PROC_SLEEPING)
            && (proc_table[parent].wakeup_events & WAKEUP_EVENT_SIGNAL)
            && (proc_table[parent].wakeup_signal == sig))
          {
            //
            // The signal is not ignored from the parent process;
            // the parent process is sleeping;
            // the parent process is waiting for a signal;
            // the parent process is waiting for current signal.
            // So, just wake it up.
            //
            proc_table[parent].status         = PROC_READY;
            proc_table[parent].wakeup_events ^= WAKEUP_EVENT_SIGNAL;
            proc_table[parent].wakeup_signal  = 0;
          }
        else
          {
            //
            // All other cases, means to remove all dead children.
            //
            for (child = 1; child < PROCESS_MAX; child++)
              {
                if (   proc_table[child].ppid == parent
                    && proc_table[child].status == PROC_ZOMBIE)
                  {
                    proc_available (child);
                  }
              }
          }
        proc_sig_off (parent, sig);
      }
}
