#include <stdlib.h>
#include <ctype.h>
#include <errno.h>
#include <limits.h>
#include <stdbool.h>
//----------------------------------------------------------------------
#define isoctal(C)  ((int) (C >= '0' && C <= '7'))
//----------------------------------------------------------------------
long int
strtol (const char *restrict string, char **restrict endptr, int base)
{
    int      i;
    int      sign = +1;
    long int number;
    long int previous;
    int      digit;
    //
    bool     flag_prefix_oct = 0;
    bool     flag_prefix_exa = 0;
    bool     flag_prefix_dec = 0;
    //
    // Check base and string.
    //
    if (base < 0
     || base > 36
     || base == 1               // With base 1 cannot do anything.
     || string == NULL
     || string[0] == 0)
      {
        if (endptr != NULL) *endptr = string;
        errset (EINVAL);                // Invalid argument.
        return ((long int) 0);
      }
    //
    // Eat initial spaces.
    //
    for (i = 0; isspace (string[i]); i++)
      {
        ;
      }
    //
    // Check sign.
    //
    if (string[i] == '+')
      {
        sign = +1;
        i++;
      }
    else if (string[i] == '-')
      {
        sign = -1;
        i++;
      }
    //
    // Check for prefix.
    //
    if (string[i] == '0')
      {
        if (string[i+1] == 'x' || string[i+1] == 'X')
          {
            flag_prefix_exa = 1;
          }
        else if (isoctal (string[i+1]))
          {
            flag_prefix_oct = 1;
          }
        else
          {
            flag_prefix_dec = 1;
          }
      }
    else if (isdigit (string[i]))
      {
        flag_prefix_dec = 1;
      }
    //
    // Check compatibility with requested base.
    //
    if (flag_prefix_exa)
      {
        //
        // At the moment, there is a zero and a `x'. Might be exadecimal,
        // or might be a number base 33 or more.
        //
        if (base == 0)
          {
            base = 16;
          }
        else if (base == 16)
          {
            ;   // Ok.
          }
        else if (base >= 33)
          {
            ;   // Ok.
          }
        else
          {
            //
            // Incompatible sequence: only the initial zero is reported.
            //
            if (endptr != NULL) *endptr = &string[i+1];
            return ((long int) 0);
          }
        //
        // Move on, after the `0x' prefix.
        //
        i += 2;
      }
    //
    if (flag_prefix_oct)
      {
        //
        // There is a zero and a digit.
        //
        if (base == 0)
          {
            base = 8;
          }
        //
        // Move on, after the `0' prefix.
        //
        i += 1;
      }
    //
    if (flag_prefix_dec)
      {
        if (base == 0)
          {
            base = 10;
          }
      }
    //
    // Scan the string.
    //
    for (number = 0; string[i] != 0; i++)
      {
        if      (string[i] >= '0' && string[i] <= '9')
          {
            digit = string[i] - '0';
          }
        else if (string[i] >= 'A' && string[i] <= 'Z')
          {
            digit = string[i] - 'A' + 10;
          }
        else if (string[i] >= 'a' && string[i] <= 'z')
          {
            digit = string[i] - 'a' + 10;
          }
        else
          {
            //
            // This is an out of range digit.
            //
            digit = 999;
          }
        //
        // Give a sign to the digit.
        //
        digit *= sign;
        //
        // Compare with the base.
        //
        if (base > (digit * sign))
          {
            //
            // Check if the current digit can be safely computed.
            //
            previous = number;
            number  *= base;
            number  += digit;
            if (number / base != previous)
              {
                //
                // Out of range.
                //
                if (endptr != NULL) *endptr = &string[i+1];
                errset (ERANGE);        // Result too large.
                if (sign > 0)
                  {
                    return (LONG_MAX);
                  }
                else
                  {
                    return (LONG_MIN);
                  }
              }
          }
        else
          {
            if (endptr != NULL) *endptr = &string[i];
            return (number);
          }
      }
    //
    // The string is finished.
    //
    if (endptr != NULL) *endptr = &string[i];
    //
    return (number);
}
