#ifndef _SYS_OS16_H
#define _SYS_OS16_H      1
//----------------------------------------------------------------------
// This file contains all the declarations that don't have a better
// place inside standard headers files. Even declarations related to
// device numbers and system calls is contained here.
//----------------------------------------------------------------------
// Please remember that system calls should never be used (called)
// inside the kernel code, because system calls cannot be nested for
// the os16 simple architecture!
// If a particular function is necessary inside the kernel, that usually
// is made by a system call, an appropriate k_...() function must be
// made, to avoid the problem.
//----------------------------------------------------------------------

#include <sys/types.h>
#include <sys/stat.h>
#include <stdint.h>
#include <signal.h>
#include <limits.h>
#include <stdio.h>
#include <stdint.h>
#include <stddef.h>
#include <const.h>
#include <restrict.h>
#include <stdarg.h>
//----------------------------------------------------------------------
// Device numbers.
//----------------------------------------------------------------------
#define DEV_UNDEFINED_MAJOR     0x00
#define DEV_UNDEFINED           0x0000
#define DEV_MEM_MAJOR           0x01
#define DEV_MEM                 0x0101
#define DEV_NULL                0x0102
#define DEV_PORT                0x0103
#define DEV_ZERO                0x0104
#define DEV_TTY_MAJOR           0x02
#define DEV_TTY                 0x0200
#define DEV_DSK_MAJOR           0x03
#define DEV_DSK0                0x0300
#define DEV_DSK1                0x0301
#define DEV_DSK2                0x0302
#define DEV_DSK3                0x0303
#define DEV_KMEM_MAJOR          0x04
#define DEV_KMEM_PS             0x0401
#define DEV_KMEM_MMP            0x0402
#define DEV_KMEM_SB             0x0403
#define DEV_KMEM_INODE          0x0404
#define DEV_KMEM_FILE           0x0405
#define DEV_CONSOLE_MAJOR       0x05
#define DEV_CONSOLE             0x05FF
#define DEV_CONSOLE0            0x0500
#define DEV_CONSOLE1            0x0501
#define DEV_CONSOLE2            0x0502
#define DEV_CONSOLE3            0x0503
#define DEV_CONSOLE4            0x0504
//----------------------------------------------------------------------
// Current segments.
//----------------------------------------------------------------------
uint16_t _seg_i (void);
uint16_t _seg_d (void);
uint16_t _cs    (void);
uint16_t _ds    (void);
uint16_t _ss    (void);
uint16_t _es    (void);
uint16_t _sp    (void);
uint16_t _bp    (void);
#define seg_i() ((unsigned int) _seg_i ())
#define seg_d() ((unsigned int) _seg_d ())
#define cs()    ((unsigned int) _cs ())
#define ds()    ((unsigned int) _ds ())
#define ss()    ((unsigned int) _ss ())
#define es()    ((unsigned int) _es ())
#define sp()    ((unsigned int) _sp ())
#define bp()    ((unsigned int) _bp ())
//----------------------------------------------------------------------
#define min(a, b) (a < b ? a : b)
#define max(a, b) (a > b ? a : b)
//----------------------------------------------------------------------
#define INPUT_LINE_HIDDEN 0
#define INPUT_LINE_ECHO   1
#define INPUT_LINE_STARS  2
//----------------------------------------------------------------------
#define MOUNT_DEFAULT     0  // Default mount options.
#define MOUNT_RO          1  // Read only mount option.
//----------------------------------------------------------------------
#define SYS_0                    0  // Nothing to do.
#define SYS_CHDIR                1
#define SYS_CHMOD                2
#define SYS_CLOCK                3
#define SYS_CLOSE                4
#define SYS_EXEC                 5
#define SYS_EXIT                 6  // [1] see below.
#define SYS_FCHMOD               7
#define SYS_FORK                 8
#define SYS_FSTAT                9
#define SYS_KILL                10
#define SYS_LSEEK               11
#define SYS_MKDIR               12
#define SYS_MKNOD               13
#define SYS_MOUNT               14
#define SYS_OPEN                15
#define SYS_PGRP                16
#define SYS_READ                17
#define SYS_SETEUID             18
#define SYS_SETUID              19
#define SYS_SIGNAL              20
#define SYS_SLEEP               21
#define SYS_STAT                22
#define SYS_TIME                23
#define SYS_UAREA               24
#define SYS_UMASK               25
#define SYS_UMOUNT              26
#define SYS_WAIT                27
#define SYS_WRITE               28
#define SYS_ZPCHAR              29  // [2] see below.
#define SYS_ZPSTRING            30  // [2]
#define SYS_CHOWN               31
#define SYS_DUP                 33
#define SYS_DUP2                34
#define SYS_LINK                35
#define SYS_UNLINK              36
#define SYS_FCNTL               37
#define SYS_STIME               38
#define SYS_FCHOWN              39
//
// [1] The files `crt0...' need to know the value used for the
//     exit system call. If this value is modified, all the file
//     `crt0...' have also to be modified the same way.
//
// [2] These system calls were developed at the beginning, when no
//     standard I/O was available. They are to be considered as a
//     last resort for debugging purposes.
//
//----------------------------------------------------------------------
typedef struct {
    char         path[PATH_MAX];
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_chdir_t;
//----------------------------------------------------------------------
typedef struct {
    char         path[PATH_MAX];
    mode_t       mode;
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_chmod_t;
//----------------------------------------------------------------------
typedef struct {
    char         path[PATH_MAX];
    uid_t        uid;
    uid_t        gid;
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_chown_t;
//----------------------------------------------------------------------
typedef struct {
    clock_t      ret;
} sysmsg_clock_t;
//----------------------------------------------------------------------
typedef struct {
    int          fdn;
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_close_t;
//----------------------------------------------------------------------
typedef struct {
    int          fdn_old;
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_dup_t;
//----------------------------------------------------------------------
typedef struct {
    int          fdn_old;
    int          fdn_new;
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_dup2_t;
//----------------------------------------------------------------------
typedef struct {
    char         path[PATH_MAX];
    int          argc;
    int          envc;
    char         arg_data[ARG_MAX/2];
    char         env_data[ARG_MAX/2];
    uid_t        uid;
    uid_t        euid;
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_exec_t;
//----------------------------------------------------------------------
typedef struct {
    int 	 status;
} sysmsg_exit_t;
//----------------------------------------------------------------------
typedef struct {
    int          fdn;
    mode_t       mode;
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_fchmod_t;
//----------------------------------------------------------------------
typedef struct {
    int          fdn;
    uid_t        uid;
    uid_t        gid;
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_fchown_t;
//----------------------------------------------------------------------
typedef struct {
    int          fdn;
    int          cmd;
    int          arg;
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_fcntl_t;
//----------------------------------------------------------------------
typedef struct {
    pid_t        ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_fork_t;
//----------------------------------------------------------------------
typedef struct {
    int          fdn;
    struct stat  stat;
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_fstat_t;
//----------------------------------------------------------------------
typedef struct {
    pid_t        pid;
    int          signal;
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_kill_t;
//----------------------------------------------------------------------
typedef struct {
    char         path_old[PATH_MAX];
    char         path_new[PATH_MAX];
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_link_t;
//----------------------------------------------------------------------
typedef struct {
    int          fdn;
    off_t        offset;
    int          whence;
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_lseek_t;
//----------------------------------------------------------------------
typedef struct {
    char         path[PATH_MAX];
    mode_t       mode;
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_mkdir_t;
//----------------------------------------------------------------------
typedef struct {
    char         path[PATH_MAX];
    mode_t       mode;
    dev_t        device;
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_mknod_t;
//----------------------------------------------------------------------
typedef struct {
    char         path_dev[PATH_MAX];
    char         path_mnt[PATH_MAX];
    int          options;
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_mount_t;
//----------------------------------------------------------------------
typedef struct {
    char         path[PATH_MAX];
    int          flags;
    mode_t       mode;
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_open_t;
//----------------------------------------------------------------------
typedef struct {
    int          fdn;
    char         buffer[BUFSIZ];
    size_t       count;
    int          eof;
    ssize_t      ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_read_t;
//----------------------------------------------------------------------
typedef struct {
    uid_t        euid;
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_seteuid_t;
//----------------------------------------------------------------------
typedef struct {
    uid_t        uid;
    uid_t        euid;
    uid_t        suid;
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_setuid_t;
//----------------------------------------------------------------------
typedef struct {
    sighandler_t handler;
    int          signal;
    sighandler_t ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_signal_t;
//----------------------------------------------------------------------
#define WAKEUP_EVENT_SIGNAL  1    // 1, 2, 4, 8, 16,...
#define WAKEUP_EVENT_TIMER   2    // so that can be `OR' combined.
#define WAKEUP_EVENT_TTY     4    //
typedef struct {
    int          events;
    int          signal;
    unsigned int seconds;
    time_t       ret;
} sysmsg_sleep_t;
//----------------------------------------------------------------------
typedef struct {
    char         path[PATH_MAX];
    struct stat  stat;
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_stat_t;
//----------------------------------------------------------------------
typedef struct {
    time_t       ret;
} sysmsg_time_t;
//----------------------------------------------------------------------
typedef struct {
    time_t       timer;
    int          ret;
} sysmsg_stime_t;
//----------------------------------------------------------------------
typedef struct {
    uid_t        uid;         // Read user ID.
    uid_t        euid;        // Effective user ID.
    uid_t        suid;        // Saved user ID.
    pid_t        pid;         // Process ID.
    pid_t        ppid;        // Parent PID.
    pid_t        pgrp;        // Process group.
    mode_t       umask;	      // Access permission mask.
    char         path_cwd[PATH_MAX];
} sysmsg_uarea_t;
//----------------------------------------------------------------------
typedef struct {
    mode_t       umask;
    mode_t       ret;
} sysmsg_umask_t;
//----------------------------------------------------------------------
typedef struct {
    char         path_mnt[PATH_MAX];
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_umount_t;
//----------------------------------------------------------------------
typedef struct {
    char         path[PATH_MAX];
    int          ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_unlink_t;
//----------------------------------------------------------------------
typedef struct {
    int          status;
    pid_t        ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_wait_t;
//----------------------------------------------------------------------
typedef struct {
    int          fdn;
    char         buffer[BUFSIZ];
    size_t       count;
    ssize_t      ret;
    int          errno;
    int          errln;
    char         errfn[PATH_MAX];
} sysmsg_write_t;
//----------------------------------------------------------------------
typedef struct {
    char         c;
} sysmsg_zpchar_t;
//----------------------------------------------------------------------
typedef struct {
    char         string[BUFSIZ];
} sysmsg_zpstring_t;
//----------------------------------------------------------------------
void  heap_clear   (void);
int   heap_min     (void);
void  input_line   (char *line, char *prompt, size_t size, int type);
int   mount        (const char *path_dev, const char *path_mnt,
                   int options);
int   namep        (const char *name, char *path, size_t size);
void  process_info (void);
void  sys          (int syscallnr, void *message, size_t size);
int   umount       (const char *path_mnt);
void  z_perror     (const char *string);
int   z_printf     (const char *restrict format, ...);
int   z_putchar    (int c);
int   z_puts       (const char *string);
int   z_vprintf    (const char *restrict format, va_list arg);
//int  z_vsprintf   (char *restrict string, const char *restrict format,
//                   va_list arg);
//----------------------------------------------------------------------

#endif
