#include <sys/os16.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
//----------------------------------------------------------------------
int
namep (const char *name, char *path, size_t size)
{
    char  command[PATH_MAX];
    char *env_path;
    int   p;            // Index used inside the path environment.
    int   c;            // Index used inside the command string.
    int   status;
    //
    // Check for valid input.
    //
    if (name == NULL || name[0] == 0 || path == NULL || name == path)
      {
        errset (EINVAL);                // Invalid argument.
        return (-1);
      }
    //
    // Check if the original command contains at least a `/'. Otherwise
    // a scan for the environment variable `PATH' must be done.
    //
    if (strchr (name, '/') == NULL)
      {
        //
        // Ok: no `/' there. Get the environment variable `PATH'.
        //
        env_path = getenv ("PATH");
        if (env_path == NULL)
          {
            //
            // There is no `PATH' environment value.
            //
            errset (ENOENT);            // No such file or directory.
            return (-1);
          }
        //
        // Scan paths and try to find a file with that name.
        //
        for (p = 0; env_path[p] != 0;)
          {
            for (c = 0;
                 c < (PATH_MAX - strlen(name) - 2) &&
                 env_path[p] != 0                  &&
                 env_path[p] != ':';
                 c++, p++)
              {
                command[c] = env_path[p];
              }
            //
            // If the loop is ended because the command array does not
            // have enough room for the full path, then must return an
            // error.
            //
            if (env_path[p] != ':' && env_path[p] != 0)
              {
                errset (ENAMETOOLONG);  // Filename too long.
                return (-1);
              }
            //
            // The command array has enough space. At index `c' must
            // place a zero, to terminate current string.
            //
            command[c] = 0;
            //
            // Add the rest of the path.
            //
            strcat (command, "/");
            strcat (command, name);
            //
            // Verify to have something with that full path name.
            //
            status = access (command, F_OK);
            if (status == 0)
              {
                //
                // Verify to have enough room inside the destination
                // path.
                //
                if (strlen (command) >= size)
                  {
                    //
                    // Sorry: too big. There must be room also for
                    // the string termination null character.
                    //
                    errset (ENAMETOOLONG);      // Filename too long.
                    return (-1);
                  }
                //
                // Copy the path and return.
                //
                strncpy (path, command, size);
                return (0);
              }
            //
            // That path was not good: try again. But before returning
            // to the external loop, must verify if `p' is to be
            // incremented, after a `:', because the external loop
            // does not touch the index `p',
            //
            if (env_path[p] == ':')
              {
                p++;
              }
          }
        //
        // At this point, there is no match with the paths.
        //
        errset (ENOENT);                // No such file or directory.
        return (-1);
      }
    //
    // At this point, a path was given and the environment variable
    // `PATH' was not scanned. Just copy the same path. But must verify
    // that the receiving path has enough room for it.
    //
    if (strlen (name) >= size)
      {
        //
        // Sorry: too big.
        //
        errset (ENAMETOOLONG);  // Filename too long.
        return (-1);
      }
    //
    // Ok: copy and return.
    //
    strncpy (path, name, size);
    return (0);
}
